@@ -22,7 +22,7 @@
 
 #include <iostream>
 
-#include <errno.h>
+#include <cerrno>
 
 using namespace std;
 
@@ -32,6 +32,8 @@ FSTProcessor::FSTProcessor()
   // escaped_chars chars
   escaped_chars.insert(L'[');
   escaped_chars.insert(L']');
+  escaped_chars.insert(L'{');
+  escaped_chars.insert(L'}');
   escaped_chars.insert(L'^');
   escaped_chars.insert(L'$');
   escaped_chars.insert(L'/');
@@ -41,11 +43,22 @@ FSTProcessor::FSTProcessor()
   escaped_chars.insert(L'>');
   
   caseSensitive = false;
+  dictionaryCase = false;
+  compoundDecomposition = false;
   nullFlush = false;
+  nullFlushGeneration = false;
+
+  pool = new Pool<vector<int> >(4, vector<int>(50));
+
+  initial_state = new State(pool);
+  current_state = new State(pool);
 }
 
 FSTProcessor::~FSTProcessor()
 {
+  delete current_state;
+  delete initial_state;
+  delete pool;
 }
 
 void
@@ -262,7 +275,6 @@ FSTProcessor::readPostgeneration(FILE *input)
   }
 }
 
-
 void
 FSTProcessor::skipUntil(FILE *input, FILE *output, wint_t const character)
 {
@@ -274,23 +286,36 @@ FSTProcessor::skipUntil(FILE *input, FILE *output, wint_t const character)
       return;
     }
 
-    if(val == L'\\')
+    switch(val)
     {
-      val = fgetwc_unlocked(input);
-      if(feof(input))
-      {
-	return;
-      }
-      fputwc_unlocked(L'\\', output);
-      fputwc_unlocked(val, output);
-    }
-    else if(val == character)
-    {
-      return;
-    }
-    else
-    {
-      fputwc_unlocked(val, output);
+      case L'\\':
+        val = fgetwc_unlocked(input);
+        if(feof(input))
+        {
+	  return;
+        }
+        fputwc_unlocked(L'\\', output);
+        fputwc_unlocked(val, output);
+        break;
+      
+      case L'\0':
+        fputwc_unlocked(val, output);
+        if(nullFlushGeneration)
+        {
+          fflush(output);
+        }
+        break;
+      
+      default:
+        if(val == character)
+        {
+          return;
+        }
+        else
+        {
+          fputwc_unlocked(val, output);
+        }
+        break;
     }
   }
 }
@@ -403,7 +428,7 @@ FSTProcessor::calcInitial()
     root.addTransition(0, 0, it->second.getInitial());
   }
 
-  initial_state.init(&root);
+  initial_state->init(&root);
 }
 
 bool
@@ -468,7 +493,6 @@ FSTProcessor::writeEscaped(wstring const &str, FILE *output)
   } 
 }
 
-
 void
 FSTProcessor::printWord(wstring const &sf, wstring const &lf, FILE *output)
 {
@@ -478,6 +502,15 @@ FSTProcessor::printWord(wstring const &sf, wstring const &lf, FILE *output)
   fputwc_unlocked(L'$', output);
 }
 
+void
+FSTProcessor::printWordBilingual(wstring const &sf, wstring const &lf, FILE *output)
+{
+  fputwc_unlocked(L'^', output);
+  fputws_unlocked(sf.c_str(), output);
+  fputws_unlocked(lf.c_str(), output);
+  fputwc_unlocked(L'$', output);
+}
+
 void
 FSTProcessor::printUnknownWord(wstring const &sf, FILE *output)
 {
@@ -609,6 +642,173 @@ FSTProcessor::initBiltrans()
   initGeneration();
 }
 
+wstring
+FSTProcessor::decompose(wstring w) 
+{
+        State current_state = *initial_state;
+
+        vector<vector<wstring> > elements;
+        bool firstupper = false, uppercase = false, last = false;
+        unsigned int index = 0;
+
+        if(w.length() < 7) // Heuristic: Do not try and decompound short words
+        {
+          return L"";
+        }
+        //wcerr << L"+ decompose: " << w << endl;
+
+        for (unsigned int i = 0; i < w.length(); i++) 
+        { 
+          //if(i == (w.length() - 1)) 
+          if(i == (w.length())) 
+          {
+            last = true;
+          }
+
+          State previous_state = current_state;
+
+          unsigned int val = w.at(i);
+
+          //wcerr << L"++ [" << last << L"][" << current_state.size() << L"] decompose: " << w.at(i) << endl;
+
+          if(last) 
+          {
+            previous_state = current_state;
+          }
+          else 
+          {
+            previous_state = current_state;
+
+            if (current_state.size() != 0) 
+            {
+              if (!isAlphabetic(val) && iswupper(val) && !caseSensitive) 
+              {
+                current_state.step(val, towlower(val)); 
+              } 
+              else 
+              {
+                //wcerr << L"+++ step: " << w.at(i) << endl;
+                current_state.step(val);
+              }
+            }
+          }
+          if(i == (w.length())-1) 
+          {
+            last = true;
+          }
+         
+          if (current_state.size() == 0 || last) 
+          {
+            //wcerr << L"+++ [" << last << L"][" << current_state.size() << L"]" << endl;
+            if(current_state.isFinal(all_finals)) 
+            {
+              previous_state = current_state;
+            }
+
+            if(previous_state.isFinal(all_finals)) 
+            {
+              firstupper = iswupper(w.at(0));
+              wstring result = previous_state.filterFinals(all_finals, alphabet,
+                                        escaped_chars,
+                                        uppercase, firstupper);
+
+              result = result.substr(1, result.size());
+              //wcerr << L"++++ result[" << index << L"]: " <<  result << endl;
+              vector<wstring> lfs;
+              wstring::size_type pos;
+              
+              pos = result.find(L'/');
+              if(pos == wstring::npos) 
+              {
+                lfs.push_back(result);
+              } 
+              else
+              {
+                while(pos != wstring::npos) 
+                {
+                  lfs.push_back(result.substr(0, pos));
+                  result.erase(0, pos + 1);
+                  pos = result.find(L'/');
+                }
+                lfs.push_back(result.substr(0, pos));
+              }
+              elements.push_back(lfs); // Add final analysis
+              index++;
+            }
+            else
+            {
+              return L"";
+            }
+
+            if (!last) 
+            {
+              current_state = *initial_state;
+              i--; 
+            }
+          }
+        }
+
+        //wcerr << L"+++ index: " << index << endl;
+        if(index > 3 || index < 2)  // Heuristic: Only permit binary/trinary compounds
+        {
+          return L"";
+        }
+
+        wstring lf = L"";
+
+        if(index != elements.size()) 
+        {
+          //wcerr << L"++ index != elements.size(): " << index << L" != " << elements.size() << endl;
+          return L"";
+        }
+
+        vector<wstring> first_elements = elements.at(0);
+        vector<wstring> second_elements = elements.at(1);
+
+        if(first_elements.size() == 0 || second_elements.size() == 0) 
+        {
+          //wcerr << L"++ first or second empty" << endl;
+          return L"";
+        }
+ 
+        if(index == 2) 
+        {  
+          for(unsigned int j = 0; j < first_elements.size(); j++)
+          {
+            for(unsigned int y = 0; y < second_elements.size(); y++) 
+            {
+              wstring analysis = first_elements.at(j) + L"+" + second_elements.at(y);
+              lf = lf + L"/" + analysis;
+              //wcerr << L"++++++ [" << j << L"][" << y << L"] compound_analysis: " << analysis << endl;
+            }
+          }
+        }
+        else if(index == 3)  
+        {
+          vector<wstring> third_elements = elements.at(2);
+
+          for(unsigned int j = 0; j < first_elements.size(); j++)
+          {
+            for(unsigned int k = 0; k < second_elements.size(); k++)
+            {
+              for(unsigned int y = 0; y < third_elements.size(); y++) 
+              {
+                wstring analysis = first_elements.at(j) + L"+" + second_elements.at(k) + L"+" + third_elements.at(y);
+                lf = lf + L"/" + analysis;
+                //wcerr << L"++++++ [" << j << L"][" << y << L"] compound_analysis: " << analysis << endl;
+              }
+            }
+          }
+
+        } 
+        else 
+        {
+          return L"";
+        }
+        //wcerr << L"+ decompose: " << lf << endl;
+        return lf;
+}
+
 void
 FSTProcessor::analysis(FILE *input, FILE *output)
 {
@@ -620,10 +820,11 @@ FSTProcessor::analysis(FILE *input, FILE *output)
   bool last_incond = false;
   bool last_postblank = false;
   bool last_preblank = false;
-  State current_state = initial_state;
+  State current_state = *initial_state;
   wstring lf = L"";
   wstring sf = L"";
   int last = 0;
+  bool firstupper = false, uppercase = false;
 
   while(wchar_t val = readAnalysis(input))
   {
@@ -632,8 +833,11 @@ FSTProcessor::analysis(FILE *input, FILE *output)
     {
       if(current_state.isFinal(inconditional))
       {
-        bool firstupper = iswupper(sf[0]);
-        bool uppercase = firstupper && iswupper(sf[sf.size()-1]);
+        if(!dictionaryCase)
+        {
+          firstupper = iswupper(sf[0]);
+          uppercase = firstupper && iswupper(sf[sf.size()-1]);
+        }
 
         lf = current_state.filterFinals(all_finals, alphabet,
                                         escaped_chars,
@@ -643,8 +847,11 @@ FSTProcessor::analysis(FILE *input, FILE *output)
       }
       else if(current_state.isFinal(postblank))
       {
-        bool firstupper = iswupper(sf[0]);
-        bool uppercase = firstupper && iswupper(sf[sf.size()-1]);
+        if(!dictionaryCase)
+        {
+          firstupper = iswupper(sf[0]);
+          uppercase = firstupper && iswupper(sf[sf.size()-1]);
+        }
 
         lf = current_state.filterFinals(all_finals, alphabet,
                                         escaped_chars,
@@ -654,8 +861,11 @@ FSTProcessor::analysis(FILE *input, FILE *output)
       }
       else if(current_state.isFinal(preblank))
       {
-        bool firstupper = iswupper(sf[0]);
-        bool uppercase = firstupper && iswupper(sf[sf.size()-1]);
+        if(!dictionaryCase)
+        {
+          firstupper = iswupper(sf[0]);
+          uppercase = firstupper && iswupper(sf[sf.size()-1]);
+        }
 
         lf = current_state.filterFinals(all_finals, alphabet,
                                         escaped_chars,
@@ -665,8 +875,11 @@ FSTProcessor::analysis(FILE *input, FILE *output)
       }
       else if(!isAlphabetic(val))
       {
-        bool firstupper = iswupper(sf[0]);
-        bool uppercase = firstupper && iswupper(sf[sf.size()-1]);
+        if(!dictionaryCase)
+        {
+          firstupper = iswupper(sf[0]);
+          uppercase = firstupper && iswupper(sf[sf.size()-1]);
+        }
 
         lf = current_state.filterFinals(all_finals, alphabet, 
                                         escaped_chars, 
@@ -761,7 +974,24 @@ FSTProcessor::analysis(FILE *input, FILE *output)
         else
         { 
           input_buffer.back(1+(size-limit));
-          printUnknownWord(sf.substr(0, limit), output);
+          wstring unknown_word = sf.substr(0, limit);
+          if(compoundDecomposition) 
+          {
+            wstring compound = L"";
+            compound = decompose(unknown_word);
+            if(compound != L"") 
+            {
+              printWord(unknown_word, compound, output);
+            }
+            else 
+            {
+              printUnknownWord(unknown_word, output);
+            }
+          } 
+          else 
+          {
+            printUnknownWord(unknown_word, output);
+          }
         }
       }
       else if(lf == L"")
@@ -777,7 +1007,25 @@ FSTProcessor::analysis(FILE *input, FILE *output)
         else
         { 
           input_buffer.back(1+(size-limit));
-          printUnknownWord(sf.substr(0, limit), output);
+          wstring unknown_word = sf.substr(0, limit);
+          if(compoundDecomposition) 
+          {
+            wstring compound = L"";
+            compound = decompose(unknown_word);
+            if(compound != L"") 
+            {
+              printWord(unknown_word, compound, output);
+            }
+            else 
+            {
+              printUnknownWord(unknown_word, output);
+            }
+          } 
+          else 
+          {
+            printUnknownWord(unknown_word, output);
+          }
+
         }
       }
       else
@@ -788,7 +1036,7 @@ FSTProcessor::analysis(FILE *input, FILE *output)
         input_buffer.back(1);
       }
 	
-      current_state = initial_state;
+      current_state = *initial_state;
       lf = L"";
       sf = L"";
       last_incond = false;
@@ -822,6 +1070,8 @@ FSTProcessor::generation_wrapper_null_flush(FILE *input, FILE *output,
                                             GenerationMode mode)
 {
   setNullFlush(false);
+  nullFlushGeneration = true;
+  
   while(!feof(input)) 
   {
     generation(input, output, mode);
@@ -869,7 +1119,7 @@ FSTProcessor::transliteration_wrapper_null_flush(FILE *input, FILE *output)
 void
 FSTProcessor::tm_analysis(FILE *input, FILE *output)
 {
-  State current_state = initial_state;
+  State current_state = *initial_state;
   wstring lf = L"";
   wstring sf = L"";
   int last = 0;
@@ -1012,7 +1262,7 @@ FSTProcessor::tm_analysis(FILE *input, FILE *output)
         input_buffer.back(1);
       }
 	
-      current_state = initial_state;
+      current_state = *initial_state;
       lf = L"";
       sf = L"";
     }
@@ -1031,7 +1281,7 @@ FSTProcessor::generation(FILE *input, FILE *output, GenerationMode mode)
     generation_wrapper_null_flush(input, output, mode);
   }
 
-  State current_state = initial_state;
+  State current_state = *initial_state;
   wstring sf = L"";
  
   outOfWord = false;
@@ -1080,10 +1330,22 @@ FSTProcessor::generation(FILE *input, FILE *output, GenerationMode mode)
         bool uppercase = sf.size() > 1 && iswupper(sf[1]);
         bool firstupper= iswupper(sf[0]);
 
+        if(mode == gm_tagged)
+        {
+	  fputwc_unlocked(L'^', output);
+        }
+
         fputws_unlocked(current_state.filterFinals(all_finals, alphabet,
                                                   escaped_chars,
                                                   uppercase, firstupper).substr(1).c_str(),
 						  output);
+        if(mode == gm_tagged)
+        {
+	  fputwc_unlocked(L'/', output);
+	  fputws_unlocked(sf.c_str(), output);
+	  fputwc_unlocked(L'$', output);
+        }
+
       }
       else
       {
@@ -1103,7 +1365,7 @@ FSTProcessor::generation(FILE *input, FILE *output, GenerationMode mode)
         }
       }
   
-      current_state = initial_state;
+      current_state = *initial_state;
       sf = L"";
     }
     else if(iswspace(val) && sf.size() == 0)
@@ -1141,10 +1403,11 @@ FSTProcessor::postgeneration(FILE *input, FILE *output)
   }
 
   bool skip_mode = true;
-  State current_state = initial_state;
+  State current_state = *initial_state;
   wstring lf = L"";
   wstring sf = L"";
   int last = 0;
+  set<wchar_t> empty_escaped_chars;
 
   while(wchar_t val = readPostgeneration(input))
   {
@@ -1176,7 +1439,7 @@ FSTProcessor::postgeneration(FILE *input, FILE *output)
         bool firstupper = iswupper(sf[1]);
         bool uppercase = sf.size() > 1 && firstupper && iswupper(sf[2]);
         lf = current_state.filterFinals(all_finals, alphabet,
-					escaped_chars,
+					empty_escaped_chars,
 					uppercase, firstupper, 0);
 
         // case of the beggining of the next word
@@ -1286,7 +1549,7 @@ FSTProcessor::postgeneration(FILE *input, FILE *output)
 	  }	  
 	}
 
-	current_state = initial_state;
+	current_state = *initial_state;
 	lf = L"";
 	sf = L"";
 	skip_mode = true;
@@ -1306,7 +1569,7 @@ FSTProcessor::transliteration(FILE *input, FILE *output)
     transliteration_wrapper_null_flush(input, output);
   }
 
-  State current_state = initial_state;
+  State current_state = *initial_state;
   wstring lf = L"";
   wstring sf = L"";
   int last = 0;
@@ -1322,7 +1585,7 @@ FSTProcessor::transliteration(FILE *input, FILE *output)
       if(!lf.empty()) 
       {
         fputws_unlocked(lf.substr(1).c_str(), output);
-        current_state = initial_state;
+        current_state = *initial_state;
         lf = L"";
         sf = L"";
       }
@@ -1379,7 +1642,7 @@ FSTProcessor::transliteration(FILE *input, FILE *output)
             fputwc_unlocked(val, output);
           }
         }
-        current_state = initial_state;
+        current_state = *initial_state;
         lf = L"";
         sf = L"";
       }
@@ -1389,10 +1652,179 @@ FSTProcessor::transliteration(FILE *input, FILE *output)
   flushBlanks(output);
 }
 
+wstring
+FSTProcessor::biltransfull(wstring const &input_word, bool with_delim)
+{
+  State current_state = *initial_state;
+  wstring result = L"";
+  unsigned int start_point = 1;
+  unsigned int end_point = input_word.size()-2;
+  wstring queue = L"";
+  bool mark = false;
+  
+  if(with_delim == false)
+  {
+    start_point = 0;
+    end_point = input_word.size()-1;
+  }
+
+  if(input_word[start_point] == L'*')
+  {
+    return input_word;
+  }
+  
+  if(input_word[start_point] == L'=')
+  {
+    start_point++;
+    mark = true;
+  }
+  
+  bool firstupper = iswupper(input_word[start_point]);
+  bool uppercase = firstupper && iswupper(input_word[start_point+1]);
+
+  for(unsigned int i = start_point; i <= end_point; i++)
+  {
+    int val;
+    wstring symbol = L"";
+ 
+    if(input_word[i] == L'\\')
+    {
+      i++;
+      val = static_cast<int>(input_word[i]);
+    }
+    else if(input_word[i] == L'<')
+    {
+      symbol = L'<';
+      for(unsigned int j = i + 1; j <= end_point; j++)
+      {
+	symbol += input_word[j];
+	if(input_word[j] == L'>')
+	{
+	  i = j;
+	  break;
+	}
+      }
+      val = alphabet(symbol);
+    }
+    else
+    {
+      val = static_cast<int>(input_word[i]);
+    }
+    if(current_state.size() != 0)
+    {
+      if(!alphabet.isTag(val) && iswupper(val) && !caseSensitive)
+      {
+	current_state.step(val, towlower(val));
+      }
+      else
+      {
+	current_state.step(val);
+      }
+    }
+    if(current_state.isFinal(all_finals)) 
+    {
+      result = current_state.filterFinals(all_finals, alphabet,
+                                         escaped_chars,
+                                         uppercase, firstupper, 0);
+      if(with_delim)
+      {      
+        if(mark)
+        {
+          result = L"^="+result.substr(1);
+        }
+        else
+        {
+          result[0] = L'^';
+        }
+      }
+      else
+      {
+        if(mark)
+        {
+          result = L"=" + result.substr(1);
+        }
+        else
+        {
+          result = result.substr(1);
+        }
+      }
+    }
+    
+    if(current_state.size() == 0)
+    { 
+      if(symbol != L"" && result != L"")
+      {
+        queue.append(symbol);
+      }
+      else
+      {
+	// word is not present
+        if(with_delim)
+	{
+          result = L"^@" + input_word.substr(1);  
+	}
+        else
+	{
+          result = L"@" + input_word;
+	}
+        return result;  
+      }      
+    }
+  }
+
+  if(start_point < (end_point - 3)) 
+  {
+    return L"^$";
+  }
+  // attach unmatched queue automatically
+
+  if(queue != L"")
+  {
+    wstring result_with_queue = L"";    
+    bool multiple_translation = false;
+    for(unsigned int i = 0, limit = result.size(); i != limit; i++)
+    {
+      switch(result[i])
+      {
+        case L'\\':
+          result_with_queue += L'\\';
+	  i++;
+          break;
+     
+        case L'/':
+          result_with_queue.append(queue);
+	  multiple_translation = true;
+	  break;
+    
+        default:
+          break;
+      }
+      result_with_queue += result[i];
+    }
+    result_with_queue.append(queue);
+
+    if(with_delim)
+    {
+      result_with_queue += L'$';
+    }
+    return result_with_queue;
+  }
+  else
+  {
+    if(with_delim)
+    {
+      result += L'$';
+    }
+    return result;
+  }
+}
+
+
+
 wstring
 FSTProcessor::biltrans(wstring const &input_word, bool with_delim)
 {
-  State current_state = initial_state;
+  State current_state = *initial_state;
   wstring result = L"";
   unsigned int start_point = 1;
   unsigned int end_point = input_word.size()-2;
@@ -1552,10 +1984,144 @@ FSTProcessor::biltrans(wstring const &input_word, bool with_delim)
   }
 }
 
+void
+FSTProcessor::bilingual_wrapper_null_flush(FILE *input, FILE *output)
+{
+  setNullFlush(false);
+  nullFlushGeneration = true;
+  
+  while(!feof(input)) 
+  {
+    bilingual(input, output);
+    fputwc_unlocked(L'\0', output);
+    int code = fflush(output);
+    if(code != 0) 
+    {
+        wcerr << L"Could not flush output " << errno << endl;
+    }
+  }
+}
+
+wstring
+FSTProcessor::compose(wstring const &lexforms, wstring const &queue) const
+{
+  wstring result = L"";
+  
+  for(unsigned int i = 1; i< lexforms.size(); i++)
+  {
+    if(lexforms[i] == L'\\')
+    {
+      result += L'\\';
+      i++;
+    }
+    else if(lexforms[i] == L'/')
+    {
+      result.append(queue);
+    }
+    result += lexforms[i];
+  }
+  
+  return L"/" + result + queue;
+}
+
+void
+FSTProcessor::bilingual(FILE *input, FILE *output)
+{
+  if(getNullFlush())
+  {
+    bilingual_wrapper_null_flush(input, output);
+  }
+
+  State current_state = *initial_state;
+  wstring sf = L"";
+  wstring queue = L"";
+  wstring result = L"";
+  
+  outOfWord = false;
+ 
+  skipUntil(input, output, L'^');
+  int val;
+
+  while((val = readGeneration(input, output)) != 0x7fffffff)
+  {
+    if(val == L'$' && outOfWord)
+    {
+      if(sf[0] == L'*')
+      {
+        printWordBilingual(sf, L"/"+sf, output);
+      }
+      else if(result != L"")
+      {
+        printWordBilingual(sf, compose(result, queue), output);
+      }
+      else
+      {
+        printWordBilingual(sf, L"/@"+sf, output);
+      }
+  
+      queue = L"";
+      result = L"";
+      current_state = *initial_state;
+      sf = L"";
+    }
+    else if(iswspace(val) && sf.size() == 0)
+    {
+      // do nothing
+    }
+    else if(sf.size() > 0 && sf[0] == L'*')
+    {
+      if(escaped_chars.find(val) != escaped_chars.end())
+      {
+        sf += L'\\';
+      }
+      alphabet.getSymbol(sf, val);
+    }
+    else
+    {
+      if(escaped_chars.find(val) != escaped_chars.end())
+      {
+        sf += L'\\';
+      }
+      alphabet.getSymbol(sf,val);
+      if(current_state.size() != 0)
+      {
+	if(!alphabet.isTag(val) && iswupper(val) && !caseSensitive)
+	{
+	  current_state.step(val, towlower(val));
+	}
+	else
+	{
+	  current_state.step(val);
+	}
+      }
+      if(current_state.isFinal(all_finals))
+      {
+        bool uppercase = sf.size() > 1 && iswupper(sf[1]);
+        bool firstupper= iswupper(sf[0]);
+
+        result = current_state.filterFinals(all_finals, alphabet,
+                                            escaped_chars,
+                                            uppercase, firstupper, 0);     
+      }
+      if(current_state.size() == 0 && result != L"")
+      {
+        if(alphabet.isTag(val))
+        {
+          alphabet.getSymbol(queue, val);
+        }
+        else
+        {
+          result = L"";
+        }
+      }
+    }
+  }
+}
+
 pair<wstring, int>
 FSTProcessor::biltransWithQueue(wstring const &input_word, bool with_delim)
 {
-  State current_state = initial_state;
+  State current_state = *initial_state;
   wstring result = L"";
   unsigned int start_point = 1;
   unsigned int end_point = input_word.size()-2;
@@ -1718,7 +2284,7 @@ FSTProcessor::biltransWithQueue(wstring const &input_word, bool with_delim)
 wstring
 FSTProcessor::biltransWithoutQueue(wstring const &input_word, bool with_delim)
 {
-  State current_state = initial_state;
+  State current_state = *initial_state;
   wstring result = L"";
   unsigned int start_point = 1;
   unsigned int end_point = input_word.size()-2;
@@ -1841,14 +2407,14 @@ FSTProcessor::biltransWithoutQueue(wstring const &input_word, bool with_delim)
 bool
 FSTProcessor::valid() const
 {
-  if(initial_state.isFinal(all_finals))
+  if(initial_state->isFinal(all_finals))
   {
     wcerr << L"Error: Invalid dictionary (hint: the left side of an entry is empty)" << endl;
     return false;
   }
   else
   {
-    State s = initial_state;
+    State s = *initial_state;
     s.step(L' ');
     if(s.size() != 0)
     {
@@ -1932,7 +2498,7 @@ FSTProcessor::SAO(FILE *input, FILE *output)
 {
   bool last_incond = false;
   bool last_postblank = false;
-  State current_state = initial_state;
+  State current_state = *initial_state;
   wstring lf = L"";
   wstring sf = L"";
   int last = 0;
@@ -2069,7 +2635,7 @@ FSTProcessor::SAO(FILE *input, FILE *output)
         input_buffer.back(1);
       }
 	
-      current_state = initial_state;
+      current_state = *initial_state;
       lf = L"";
       sf = L"";
       last_incond = false;
@@ -2095,18 +2661,36 @@ FSTProcessor::removeTags(wstring const &str)
   return str;
 }
 
+void
+FSTProcessor::setDecompoundingMode(bool const value)
+{
+  compoundDecomposition = value;
+}
+
 void
 FSTProcessor::setCaseSensitiveMode(bool const value)
 {
   caseSensitive = value;
 }
 
+void
+FSTProcessor::setDictionaryCaseMode(bool const value)
+{
+  dictionaryCase = value;
+}
+
 void
 FSTProcessor::setNullFlush(bool const value)
 {
   nullFlush = value;
 }
 
+bool
+FSTProcessor::getDecompoundingMode()
+{
+  return compoundDecomposition;
+}
+
 bool
 FSTProcessor::getNullFlush()
 {
