@@ -25,7 +25,7 @@
 #include <libgen.h>
 #include <string>
 
-#ifdef WIN32
+#ifdef _MSC_VER
 #include <io.h>
 #include <fcntl.h>
 #endif
@@ -81,7 +81,7 @@ int main(int argc, char *argv[])
       break;
   }
 
-#ifdef WIN32
+#ifdef _MSC_VER
   _setmode(_fileno(output), _O_U8TEXT);
 #endif
 
